//Formcontrols (require object.js)
var FormControls = function (params) {
	/*================
	Model
	================*/
	var defaults = {
		rangeTipClass: "range-tooltip",//滑动条弹出框
		searchClass: "inputbox-search",
		numboxClass: "numbox",//数字框
		switchClass: "switch",//开关控件
		turnClass: "turn",//开关控件
		revealAttr: "[data-input=reveal]",//密码小眼睛
		clearAttr: "[data-input=clear]",//清空按钮
		safelvlClass: "safelvl",//密码安全等级
		inputpreClass: "input-pre"//自动增高控件
	}
	params = params || {};
	for (var def in defaults) {
		if (params[def] === undefined) {
			params[def] = defaults[def];
		}
	}
	//Formcontrols
	var s = this;
	//Params
	s.params = params;

	/*================
	Method
	================*/
	//自动增高控件，创建pre元素
	s.createPreAfter = function (el) {
		var pre = document.createElement("pre");
		var span = document.createElement("span");
		pre.appendChild(span);
		pre.style.width = el.clientWidth + "px";
		el.parentNode.insertBefore(pre, el.nextSibling);
		setTimeout(function () {
			pre.style.width = el.clientWidth + "px";
		}, 100);
	}
	s.update = function () {
		s.detach();
		s.attach();
	}
	/*================
Events
================*/
	s.events = function (detach) {
		var action = detach ? "removeEventListener" : "addEventListener";
		//开关控件
		var switches = document.querySelectorAll("." + s.params.switchClass);

		for (var i = 0, swi; swi = switches[i++];) {
			swi[action]("click", s.onSwitch, false);
		}
		//开关控件
		var turns = document.querySelectorAll("." + s.params.turnClass);

		for (var i = 0, turn; turn = turns[i++];) {
			turn[action]("click", s.onSwitch, false);
		}
		//密码小眼睛
		var reveals = document.querySelectorAll(s.params.revealAttr + " > [type=password] + i");
		for (var j = 0, reveal; reveal = reveals[j++];) {
			reveal[action]("click", s.onReveal, false);
		}
		//清除按钮框
		var clears = document.querySelectorAll(s.params.clearAttr + " > input");
		var clearIcons = document.querySelectorAll(s.params.clearAttr + "> input+i");
		for (var k = 0; k < clears.length; k++) {
			clears[k][action]("input", s.onClear, false);
			if (clearIcons[k]) clearIcons[k][action]("click", s.onClearIcon, false);
		}
		//安全检测框
		var safes = document.querySelectorAll("." + s.params.safelvlClass);
		for (var l = 0, safe; safe = safes[l++];) {
			var safeInput = safe.parentNode.querySelector("input[type]");
			safeInput.safe = safe;
			safeInput[action]("input", s.onSafeLvl, false);
		}
		//拖动条
		var ranges = document.querySelectorAll("." + s.params.rangeTipClass + "+input[type=range]");
		for (var m = 0, range; range = ranges[m++];) {
			range[action]("touchstart", s.onRangeStart, false);
			range[action]("touchmove", s.onRangeMove, false);
			range[action]("input", s.onRangeMove, false);
			range[action]("touchend", s.onRangeEnd, false);
		}
		//数字框
		var numboxs = document.querySelectorAll("." + s.params.numboxClass + " input[type=number]");
		for (var n = 0, numbox; numbox = numboxs[n++];) {
			numbox.nextElementSibling[action]("click", s.onNumboxPlus, false);
			numbox.previousElementSibling[action]("click", s.onNumboxMinus, false);
		}
		//自动增高控件
		var inputpres = document.querySelectorAll("." + s.params.inputpreClass);
		for (var o = 0, inputpre; inputpre = inputpres[o++];) {
			if (!inputpre.nextElementSibling || inputpre.nextElementSibling.tagName != "PRE") {
				s.createPreAfter(inputpre);
			}
			inputpre.pre = inputpre.nextElementSibling;
			inputpre.preSpan = inputpre.pre.childNodes[0];
			inputpre[action]("input", s.onInputpre, false);
		}
		//搜索框
		var inputSearchs = document.querySelectorAll("." + s.params.searchClass + " input[type='search']");
		for (var p = 0, inputSearch; inputSearch = inputSearchs[p++];) {
			inputSearch[action]("blur", s.onBlurSearch, false);
		}
	}
	s.attach = function (event) {
		s.events();
	}
	s.detach = function (event) {
		s.events(true);
	}
	/*================
	Events Handler
	================*/
	/*开关控件*/
	s.createHiddenInput = function (name) {
		var hiddenInput = document.createElement("input");
		hiddenInput.setAttribute("type", "hidden");
		hiddenInput.setAttribute("name", name);
		return hiddenInput;
	}
	s.onSwitch = function (e) {
		var parentNode = this.parentNode;
		var name = this.getAttribute("data-name");
		var className = this.getAttribute("data-class");
		var onVal = this.getAttribute("data-on-value");
		var offVal = this.getAttribute("data-off-value");

		var hiddenInput = null;
		if (name) {
			hiddenInput = this.nextElementSibling && this.nextElementSibling.type == "hidden" && this.nextElementSibling.name == name ? this.nextElementSibling : s.createHiddenInput(name);
			if (className) hiddenInput.setAttribute("class", className);
			parentNode.insertBefore(hiddenInput, this.nextSibling);
		}

		//切换状态
		if (this.classList.contains("active")) {
			this.classList.remove("active");
			if (hiddenInput) hiddenInput.value = offVal || "0";
		} else {
			this.classList.add("active");
			if (hiddenInput) hiddenInput.value = onVal || "1";
		}
	}
	/*密码小眼睛*/
	s.onReveal = function (e) {
		var pwdInput = this.parentNode.querySelector("input[type]");
		if (this.classList.contains("active")) {
			this.classList.remove("active");
			pwdInput.type = "password";
		} else {
			this.classList.add("active");
			pwdInput.type = "text";
		}
		pwdInput.focus();
	}
	/*清除按钮框*/
	s.onClear = function (e) {
		var clearIcon = this.nextElementSibling;
		if (!clearIcon || clearIcon.tagName != "I") return;
		if (this.value.length > 0) {
			clearIcon.style.display = "block";
		} else {
			clearIcon.style.display = "none";
		}
	}
	s.onClearIcon = function (e) {
		var txtInput = this.parentNode.querySelector("input[type]");
		this.style.display = "none";
		txtInput.value = "";
		txtInput.focus();
	}
	/*安全检测框*/
	s.onSafeLvl = function (e) {
		var value = this.value
		var lvlField = this.safe
		var lvl = Object.passwordLvl(value)
		lvlField.className = lvlField.className.replace(/lvl[0-3]/, 'lvl' + lvl)
	}
	/*拖动条*/
	s.showToolTip = function (tooltip, rangeInput) {
		//当前值所占百分比
		var percent = ((rangeInput.value - rangeInput.min) / (rangeInput.max - rangeInput.min)).toFixed(2);

		//距左的位置
		var dragRange = rangeInput.clientWidth * percent;
		var offsetLeft = rangeInput.offsetLeft + dragRange - 10;
		//var currentOffsetLeft=offsetLeft-rangeInput.parentNode.offsetLeft;

		//滑块内部的实际位置
		var currentBallLeft = 28 * percent;

		//当前值的位置-滑块的位置=小球正中间的位置
		var left = offsetLeft - currentBallLeft;
		tooltip.innerHTML = rangeInput.value;
		tooltip.setAttribute("style", "display:block;left:" + left + "px");
	}
	s.rangeTooltip, s.rangeInput;
	s.onRangeStart = function (e) {
		s.rangeTooltip = this.previousElementSibling;
		s.rangeInput = this;
		s.showToolTip(s.rangeTooltip, s.rangeInput);
	}
	s.onRangeMove = function (e) {
		s.showToolTip(s.rangeTooltip, s.rangeInput);
	}
	s.onRangeEnd = function (e) {
		setTimeout(function () {
			s.rangeTooltip.style.display = "none";
		}, 1000);
	}
	/*数字框*/
	s.numboxSum = function (inputNumber, btnPlus, btnMinus, operate) {
		var min = inputNumber.getAttribute("min") || 0;
		var max = inputNumber.getAttribute("max") || 9999;
		var step = inputNumber.getAttribute("step") || 1;

		var result;

		if (operate) {//加运算
			btnMinus.disabled = false;
			result = parseInt(inputNumber.value) + parseInt(step);
			if (result >= max) {
				result = max;
				btnPlus.disabled = true;
			}
		} else {//减运算
			btnPlus.disabled = false;
			result = inputNumber.value - step;
			if (result <= min) {
				result = min;
				btnMinus.disabled = true;
			}
		}
		inputNumber.value = result;
	}
	s.onNumboxPlus = function (e) {
		var inputNumber = this.previousElementSibling;
		var btnPlus = this;
		var btnMinus = inputNumber.previousElementSibling;
		s.numboxSum(inputNumber, btnPlus, btnMinus, true);
	}
	s.onNumboxMinus = function (e) {
		var inputNumber = this.nextElementSibling;
		var btnPlus = inputNumber.nextElementSibling;
		var btnMinus = this;
		s.numboxSum(inputNumber, btnPlus, btnMinus, false);
	}
	//自动增高控件
	s.onInputpre = function (e) {
		var thisEl = e.target;
		var pre = thisEl.pre;
		var preSpan = thisEl.preSpan;
		//计算textarea高度
		preSpan.innerText = this.value;
		this.style.height = pre.clientHeight + "px";
	}
	//搜索框
	s.onBlurSearch = function (e) {
		if (e.target.value == "") {
			e.target.parentNode.classList.remove("active");
		} else {
			e.target.parentNode.classList.add("active");
		}
	}
	//初始化
	s.init = function () {
		s.attach();
	};
	s.init();
}

export default FormControls
